@@ -114,11 +114,13 @@ def evaluate(args, model, test_loader):
             losses.append(loss.item())
             accuracies.append(acc.item())
 
+    mean_accuracy = np.mean(accuracies)
     print(
         "\nTest set: Average loss: {:.4f}, Accuracy: {:.2f}%\n".format(
             np.mean(losses), np.mean(accuracies) * 100
         )
     )
+    return mean_accuracy
 
 
 def main():
@@ -222,7 +224,7 @@ def main():
     args = parser.parse_args()
     device = torch.device(args.device)
 
-    raw_dataset = load_dataset("imdb", cache_dir="imdb")
+    raw_dataset = load_dataset("imdb", cache_dir=args.data_root)
     tokenizer = BertTokenizerFast.from_pretrained("bert-base-cased")
     dataset = raw_dataset.map(
         lambda x: tokenizer(
@@ -286,9 +288,12 @@ def main():
         )
         privacy_engine.attach(optimizer)
 
+    mean_accuracy = 0
     for epoch in range(1, args.epochs + 1):
         train(args, model, train_loader, optimizer, epoch)
-        evaluate(args, model, test_loader)
+        mean_accuracy = evaluate(args, model, test_loader)
+
+    torch.save(mean_accuracy, "run_results_imdb_classification.pt")
 
 
 if __name__ == "__main__":
