@@ -2,8 +2,8 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
 from .conv import compute_conv_grad_sample  # noqa
-from .dp_lstm import compute_lstm_linear_grad_sample  # noqa
 from .dp_multihead_attention import compute_sequence_bias_grad_sample  # noqa
+from .dp_rnn import compute_rnn_linear_grad_sample  # noqa
 from .embedding import compute_embedding_grad_sample  # noqa
 from .grad_sample_module import GradSampleModule
 from .group_norm import compute_group_norm_grad_sample  # noqa
