@@ -13,16 +13,15 @@ def compute_linear_grad_sample(
 ) -> None:
     """
     Computes per sample gradients for ``nn.Linear`` layer
+
     Args:
         layer: Layer
         A: Activations
         B: Backpropagations
         batch_dim: Batch dimension position
     """
-    gs = torch.einsum("n...i,n...j->n...ij", B, A)
-    create_or_extend_grad_sample(
-        layer.weight, torch.einsum("n...ij->nij", gs), batch_dim
-    )
+    gs = torch.einsum("n...i,n...j->nij", B, A)
+    create_or_extend_grad_sample(layer.weight, gs, batch_dim)
     if layer.bias is not None:
 
         create_or_extend_grad_sample(
