@@ -8,6 +8,8 @@ import torch.nn as nn
 def average_gradients(model):
     world_size = torch.distributed.get_world_size()
     for param in model.parameters():
+        if not param.requires_grad:
+            continue
         torch.distributed.all_reduce(param.grad, op=torch.distributed.ReduceOp.SUM)
         param.grad /= world_size
 
@@ -15,6 +17,13 @@ def average_gradients(model):
 class DifferentiallyPrivateDistributedDataParallel(nn.Module):
     def __init__(self, model):
         super().__init__()
+
+        # Synchronize the model
+        params = list(model.parameters())
+        with torch.no_grad():
+            for p in params:
+                torch.distributed.broadcast(p.data, 0)
+
         self.module = model
 
     def forward(self, *args, **kwargs):
